@@ -1,16 +1,23 @@
+/*
+htop - AvailableColumnsPanel.c
+(C) 2004-2011 Hisham H. Muhammad
+Released under the GNU GPL, see the COPYING file
+in the source distribution for its full text.
+*/
 
 #include "AvailableColumnsPanel.h"
-#include "Settings.h"
+
 #include "Header.h"
-#include "ScreenManager.h"
 #include "ColumnsPanel.h"
 
-#include "Panel.h"
-
-#include "debug.h"
 #include <assert.h>
+#include <stdlib.h>
+#include <ctype.h>
 
 /*{
+#include "Panel.h"
+#include "Settings.h"
+#include "ScreenManager.h"
 
 typedef struct AvailableColumnsPanel_ {
    Panel super;
@@ -48,7 +55,8 @@ static HandlerResult AvailableColumnsPanel_eventHandler(Panel* super, int ch) {
       }
       default:
       {
-         result = Panel_selectByTyping(super, ch);
+         if (isalpha(ch))
+            result = Panel_selectByTyping(super, ch);
          break;
       }
    }
