@@ -1,16 +1,23 @@
+/*
+htop - AvailableMetersPanel.c
+(C) 2004-2011 Hisham H. Muhammad
+Released under the GNU GPL, see the COPYING file
+in the source distribution for its full text.
+*/
 
 #include "AvailableMetersPanel.h"
-#include "Settings.h"
-#include "Header.h"
-#include "ScreenManager.h"
+
 #include "CPUMeter.h"
+#include "Header.h"
+#include "ListItem.h"
 
-#include "Panel.h"
-
-#include "debug.h"
 #include <assert.h>
+#include <stdlib.h>
 
 /*{
+#include "Settings.h"
+#include "Panel.h"
+#include "ScreenManager.h"
 
 typedef struct AvailableMetersPanel_ {
    Panel super;
@@ -91,10 +98,10 @@ AvailableMetersPanel* AvailableMetersPanel_new(Settings* settings, Panel* leftMe
       }
    }
    MeterType* type = &CPUMeter;
-   int processors = settings->pl->processorCount;
-   if (processors > 1) {
+   int cpus = settings->pl->cpuCount;
+   if (cpus > 1) {
       Panel_add(super, (Object*) ListItem_new("CPU average", 0));
-      for (int i = 1; i <= processors; i++) {
+      for (int i = 1; i <= cpus; i++) {
          char buffer[50];
          sprintf(buffer, "%s %d", type->uiName, i);
          Panel_add(super, (Object*) ListItem_new(buffer, i));
