@@ -1,16 +1,19 @@
 /*
-htop
+htop - HostnameMeter.c
 (C) 2004-2011 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include "HostnameMeter.h"
-#include "Meter.h"
+
+#include "CRT.h"
 
 #include <unistd.h>
 
-#include "debug.h"
+/*{
+#include "Meter.h"
+}*/
 
 int HostnameMeter_attributes[] = {
    HOSTNAME
