@@ -1,16 +1,20 @@
 /*
-htop
-(C) 2004-2010 Hisham H. Muhammad
+htop - LoadAverageMeter.c
+(C) 2004-2011 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include "LoadAverageMeter.h"
-#include "Meter.h"
+
+#include "CRT.h"
 
 #include <curses.h>
+#include <assert.h>
 
-#include "debug.h"
+/*{
+#include "Meter.h"
+}*/
 
 int LoadAverageMeter_attributes[] = {
    LOAD_AVERAGE_FIFTEEN, LOAD_AVERAGE_FIVE, LOAD_AVERAGE_ONE
@@ -20,12 +24,15 @@ int LoadMeter_attributes[] = { LOAD };
 
 static inline void LoadAverageMeter_scan(double* one, double* five, double* fifteen) {
    int activeProcs, totalProcs, lastProc;
+   *one = 0; *five = 0; *fifteen = 0;
    FILE *fd = fopen(PROCDIR "/loadavg", "r");
-   int total = fscanf(fd, "%lf %lf %lf %d/%d %d", one, five, fifteen,
-      &activeProcs, &totalProcs, &lastProc);
-   (void) total;
-   assert(total == 6);
-   fclose(fd);
+   if (fd) {
+      int total = fscanf(fd, "%32lf %32lf %32lf %32d/%32d %32d", one, five, fifteen,
+         &activeProcs, &totalProcs, &lastProc);
+      (void) total;
+      assert(total == 6);
+      fclose(fd);
+   }
 }
 
 static void LoadAverageMeter_setValues(Meter* this, char* buffer, int size) {
